// Copyright (C) 2000, Edward W. Felten.  All rights reserved.

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.io.IOException;
import java.util.Iterator;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JRadioButton;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;


public class ATM extends JFrame {
  private ObjectConnection conn;

  public ATM() {
    super("ATM");

    SecureConnection c1 = new SecureConnection();
    SecureConnection c2 = new SecureConnection(c1);
    conn = new ObjectConnection(c1);
    new AtmServer(c2);

    getContentPane().add(new CardInserter());
    pack();
    show();
  }

  class CardInserter extends Box {
    private JComboBox comboBox;

    CardInserter() {
      super(BoxLayout.Y_AXIS);

      Box b = Box.createHorizontalBox();

      b.add(new JLabel("Choose account number: "));

      comboBox = makeComboBox();
      b.add(comboBox);

      JButton but = new JButton("Insert card");
      but.addActionListener(new ActionListener() {
	public void actionPerformed(ActionEvent ev) {
	  Integer aint = (Integer)(comboBox.getSelectedItem());
	  int anum = aint.intValue();
	  AtmCard card = AtmCard.fromFile(anum);
	  getContentPane().remove(CardInserter.this);
	  getContentPane().add(new PinGetter(card));
	  pack();
	}
      });
      b.add(but);

      JButton cBut = new JButton("Close");
      cBut.addActionListener(new ActionListener() {
	public void actionPerformed(ActionEvent ev) {
	  ATM.this.dispose();
	}
      });
      b.add(cBut);
 
      add(b);
    }

    private JComboBox makeComboBox() {
      JComboBox ret = new JComboBox();
      Iterator iter = AccountDB.getIterator();
      while(iter.hasNext()){
	Account acct = (Account)(iter.next());
	ret.addItem(new Integer(acct.getNum()));
      }
      return ret;
    }
  }

  class PinGetter extends Box {
    private AtmCard card;
    private LabeledTextField pinField;

    PinGetter(AtmCard card){ 
      super(BoxLayout.Y_AXIS);

      this.card = card;

      add(new JLabel("Account number: " + card.getNum()));

      pinField = new LabeledTextField("PIN: ", "");
      pinField.addActionListener(new ActionListener() {
	public void actionPerformed(ActionEvent e) {
	  tryToFinish();
	}
      });
      add(pinField);

      Box b = Box.createHorizontalBox();
      JButton but = new JButton("OK");
      but.addActionListener(new ActionListener() {
	public void actionPerformed(ActionEvent e) {
	  tryToFinish();
	}
      });
      b.add(but);

      but = new JButton("Show card info");
      but.addActionListener(new ActionListener() {
	public void actionPerformed(ActionEvent ev) {
	  new CardInfoFrame();
	}
      });
      b.add(but);
	    
      but = new JButton("Close");
      but.addActionListener(new ActionListener() {
	public void actionPerformed(ActionEvent ev) {
	  ATM.this.dispose();
	}
      });
      b.add(but);
      add(b);
    }

    private void tryToFinish() {
      String pinText = pinField.getText();
      if(card.authenticate(pinText)){
	LoggedIn lin = new LoggedIn(card);
	getContentPane().remove(PinGetter.this);
	getContentPane().add(lin);
	pack();
      }else{
	JOptionPane.showMessageDialog(null,
				      "Incorrect PIN",
				      "Incorrect PIN",
				      JOptionPane.ERROR_MESSAGE);
	pinField.setText("");
      }
    }

    class CardInfoFrame extends JFrame {
      CardInfoFrame() {
	super("ATM Card Info");
	getContentPane().add(makeContents());
	pack();
	show();
      }

      private Component makeContents() {
	Box ret = Box.createVerticalBox();
	ret.add(new JLabel("Account number: " + card.getNum()));
	ret.add(new CheckByteField(card));
	JButton but = new JButton("Close");
	but.addActionListener(new ActionListener() {
	  public void actionPerformed(ActionEvent ev) {
	    CardInfoFrame.this.dispose();
	  }
	});
	ret.add(but);
	return ret;
      }
    }
  }

  public static void main(String[] argv) throws IOException {
    ATM atm = new ATM();
    for(;;)    Thread.yield();
  }

  class LoggedIn extends Box {
    private AtmCard   card;
    private Account   acct;
    private JLabel    balanceLabel;

    LoggedIn(AtmCard theCard) {
      super(BoxLayout.Y_AXIS);

      card = theCard;
      acct = AccountDB.get(card.getNum());

      int num = card.getNum();

      JLabel anum = new JLabel("Account num: " + num);
      add(anum);
      JLabel name = new JLabel("Owner: " + acct.getName());
      add(name);
      balanceLabel = new JLabel("Balance: unknown");
      add(balanceLabel);
	
      Box dBox = Box.createVerticalBox();
      Box wBox = Box.createVerticalBox();
      for(int i=20; i<=100; i+=20){
	dBox.add(new DwButton(true, i));
	wBox.add(new DwButton(false, i));
      }
      Box dwBox = Box.createHorizontalBox();
      dwBox.add(dBox);
      dwBox.add(wBox);
      add(dwBox);

      JButton closeButton = new JButton("Close");
      closeButton.addActionListener(new ActionListener() {
	public void actionPerformed(ActionEvent e) {
	  dispose();
	}
      });
      add(closeButton);
    }

    private void updateBalanceLabel(Money money) {
      balanceLabel.setText("Balance: " + money);
    }

    class DwButton extends JButton{
      private boolean isDep;
      private Money money;

      DwButton(boolean dep, int dollars) {
	isDep = dep;
	money = new Money(dollars, 0);
	setText((isDep ? "Deposit " : "Withdraw ") + money);
	addActionListener(new ActionListener() {
	  public void actionPerformed(ActionEvent e) {
	    if(isDep){
	      deposit(money);
	    }else{
	      try{
		withdraw(money);
	      }catch(InsufficientFundsException x){
		JOptionPane.showMessageDialog(null,
					      "Insufficient funds -- operation cancelled.",
					      "Insufficient funds",
					      JOptionPane.ERROR_MESSAGE);
	      }
	    }
	  }
	});
      }

      private void deposit(Money amount) {
	AtmCommand cmd = new AtmCommand(card.getNum(),
					AtmCommand.Deposit,
					amount);
	try{
	  conn.send(cmd);
	  Money bal = (Money)(conn.receive());
	  updateBalanceLabel(bal);
	}catch(ConnectionClosedException x){
	  System.err.println("AtmClient: connection closed unexpectedly");
	}
      }

      private void withdraw(Money amount) 
	throws InsufficientFundsException {
	AtmCommand cmd = new AtmCommand(card.getNum(),
					AtmCommand.Withdraw,
					amount);
	try{
	  conn.send(cmd);
	  Object obj = conn.receive();
	  if(obj instanceof Money){
	    updateBalanceLabel((Money)obj);
	  }else if(obj instanceof InsufficientFundsException){
	    throw (InsufficientFundsException)obj;
	  }else{
	    System.err.println("AtmClient: unexpected reply type");
	  }
	}catch(ConnectionClosedException x){
	  System.err.println("AtmClient: connection closed unexpectedly");
	}
      }
    }
  }

  class CheckByteField extends Box {
    CheckByteField(AtmCard card) {
      super(BoxLayout.X_AXIS);
      add(new JLabel("Check bytes: "));
      add(new JLabel(byteArrToString(card.getAuth())));
    }

    private String byteArrToString(byte[] arr) {
      String ret = "";
      for(int i=0; i<arr.length; ++i){
	int x = arr[i];
	if(x < 0)    x += 256;
	ret = ret 
	  + Integer.toString(x /16, 16) 
	  + Integer.toString(x % 16, 16); 
      }
      return ret;
    }
  }
}
