// Copyright (C) 2000, Edward W. Felten.  All rights reserved.

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.util.Iterator;
import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;


public class TellerUI extends JFrame {
  public TellerUI() throws IOException {
    super("Teller Terminal");
    getContentPane().add(makeContents());
    pack();
    show();
  }

  private Component makeContents() {
    Box ret = Box.createVerticalBox();

    JScrollPane scrollPane = new JScrollPane(new AccountTable());
    ret.add(scrollPane);

    Box buttonBox = Box.createHorizontalBox();
    buttonBox.add(makeNewAccountButton());
	
    JButton closeButton = new JButton("Close");
    closeButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	dispose();
      }
    });
    buttonBox.add(closeButton);

    ret.add(buttonBox);

    return ret;
  }

  private JButton makeNewAccountButton() {
    JButton ret = new JButton("New Account");
    ret.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e){
	JFrame f = new NewAccountFrame();
      }
    });
    return ret;
  }

  public static void main(String[] argv) throws IOException {
    new TellerUI();
    for(;;)    Thread.yield();
  }

  class NewAccountFrame extends JFrame {
    LabeledTextField nameTF;
    LabeledTextField pinTF;
    LabeledTextField balanceTF;

    NewAccountFrame() {
      super("New Account");
      getContentPane().add(makeContents());
      pack();
      show();
    }

    private Component makeContents() {
      Box ret = Box.createVerticalBox();

      ActionListener al = new ActionListener() {
	public void actionPerformed(ActionEvent e){
	  try{
	    String name = nameTF.getText();
	    int pin = Integer.parseInt(pinTF.getText());
	    Money bal = new Money(balanceTF.getText());
	    new Account(name, pin, bal);
	    dispose();
	  }catch(NumberFormatException x){
	    JOptionPane.showMessageDialog(null,
					  "Bad format",
					  "Bad format",
					  JOptionPane.ERROR_MESSAGE);
	  }
	}
      };

      nameTF = new LabeledTextField("Owner: ", "");
      pinTF = new LabeledTextField("PIN: ", "");
      balanceTF = new LabeledTextField("Balance: ", "0");
      nameTF.addActionListener(al);
      pinTF.addActionListener(al);
      balanceTF.addActionListener(al);
      ret.add(nameTF);
      ret.add(pinTF);
      ret.add(balanceTF);

      Box butBox = Box.createHorizontalBox();
      JButton cBut = new JButton("Create");
      cBut.addActionListener(al);
      JButton xBut = new JButton("Cancel");
      xBut.addActionListener(new ActionListener() {
	public void actionPerformed(ActionEvent e){
	  dispose();
	}
      });
      butBox.add(cBut);
      butBox.add(xBut);

      ret.add(butBox);
      return ret;
    }
  }
}
